@@ -2,21 +2,23 @@ use crate::cla::ConvergenceLayerAgent;
 use crate::CONFIG;
 use async_trait::async_trait;
 use bp7::ByteBuffer;
+use dtn7_codegen::cla;
 use hyper::{Body, Method, Request};
 use log::{debug, error};
-use std::net::SocketAddr;
+use std::{collections::HashMap, net::SocketAddr};
 
+use super::HelpStr;
+
+#[cla(http)]
 #[derive(Debug, Clone, Default, Copy)]
 pub struct HttpConvergenceLayer {
-    counter: u64,
     local_port: u16,
 }
 
 impl HttpConvergenceLayer {
-    pub fn new(port: Option<u16>) -> HttpConvergenceLayer {
+    pub fn new(_local_settings: Option<&HashMap<String, String>>) -> HttpConvergenceLayer {
         HttpConvergenceLayer {
-            counter: 0,
-            local_port: port.unwrap_or((*CONFIG.lock()).webport),
+            local_port: (*CONFIG.lock()).webport,
         }
     }
 }
@@ -58,6 +60,8 @@ impl ConvergenceLayerAgent for HttpConvergenceLayer {
     }
 }
 
+impl HelpStr for HttpConvergenceLayer {}
+
 impl std::fmt::Display for HttpConvergenceLayer {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "http:{}", self.local_port)
