@@ -4,6 +4,7 @@ use bp7::{Bundle, ByteBuffer};
 use bytes::buf::Buf;
 use bytes::{BufMut, BytesMut};
 use core::convert::TryFrom;
+use dtn7_codegen::cla;
 use futures_util::stream::StreamExt;
 use lazy_static::lazy_static;
 use log::{debug, error, info};
@@ -19,6 +20,8 @@ use tokio::io;
 use tokio::net::TcpListener;
 use tokio_util::codec::{Decoder, Encoder, Framed};
 
+use super::HelpStr;
+
 lazy_static! {
     pub static ref MTCP_CONNECTIONS: Mutex<HashMap<SocketAddr, TcpStream>> =
         Mutex::new(HashMap::new());
@@ -194,18 +197,19 @@ impl Decoder for MPDUCodec {
     }
 }
 
-#[derive(Debug, Copy, Clone)]
+#[cla(mtcp)]
+#[derive(Debug, Copy, Clone, Default)]
 pub struct MtcpConvergenceLayer {
-    counter: u64,
     local_port: u16,
 }
 
 impl MtcpConvergenceLayer {
-    pub fn new(port: Option<u16>) -> MtcpConvergenceLayer {
-        MtcpConvergenceLayer {
-            counter: 0,
-            local_port: port.unwrap_or(16162),
-        }
+    pub fn new(local_settings: Option<&HashMap<String, String>>) -> MtcpConvergenceLayer {
+        let port = local_settings
+            .and_then(|settings| settings.get("port"))
+            .and_then(|port_str| port_str.parse::<u16>().ok())
+            .unwrap_or(16162);
+        MtcpConvergenceLayer { local_port: port }
     }
     async fn handle_connection(socket: tokio::net::TcpStream) -> anyhow::Result<()> {
         let peer_addr = socket.peer_addr().unwrap();
@@ -333,6 +337,8 @@ impl ConvergenceLayerAgent for MtcpConvergenceLayer {
     }
 }
 
+impl HelpStr for MtcpConvergenceLayer {}
+
 impl std::fmt::Display for MtcpConvergenceLayer {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(f, "mtcp:{}", self.local_port)
