@@ -10,6 +10,7 @@ use bp7::{Bundle, CreationTimestamp, EndpointID};
 use dtn7_plus::client::{WsRecvData, WsSendData};
 use futures::{sink::SinkExt, stream::StreamExt};
 use log::{debug, warn};
+use parking_lot::Mutex;
 use std::collections::HashSet;
 use std::sync::Arc;
 use std::{
@@ -17,7 +18,6 @@ use std::{
     time::{Duration, Instant},
 };
 use tokio::sync::mpsc;
-use tokio::sync::Mutex;
 use tokio::time::interval;
 // Begin application agent WebSocket specific stuff
 
@@ -74,7 +74,6 @@ pub async fn handle_socket(socket: WebSocket) {
         while let Some(Ok(msg)) = receiver.next().await {
             if session2
                 .lock()
-                .await
                 .handle_message(tx2.clone(), msg)
                 .await
                 .is_err()
@@ -94,7 +93,7 @@ pub async fn handle_socket(socket: WebSocket) {
 
         loop {
             task.tick().await;
-            if Instant::now().duration_since(session2.lock().await.hb) > CLIENT_TIMEOUT {
+            if Instant::now().duration_since(session2.lock().hb) > CLIENT_TIMEOUT {
                 // heartbeat timed out
                 debug!("Websocket Client heartbeat failed, disconnecting!");
 
@@ -116,7 +115,6 @@ pub async fn handle_socket(socket: WebSocket) {
             debug!("Received bundle delivery for {}", bndl_delivery.0.id());
             if session2
                 .lock()
-                .await
                 .handle_bundle_delivery(tx2.clone(), bndl_delivery)
                 .await
                 .is_err()
@@ -133,7 +131,7 @@ pub async fn handle_socket(socket: WebSocket) {
         _ = (&mut br_task) => {hb_task.abort(); recv_task.abort(); send_task.abort();},
     };
 
-    if let Some(endpoints) = &session.lock().await.endpoints {
+    if let Some(endpoints) = &session.lock().endpoints {
         for eid in endpoints {
             if let Some(ep) = (*DTNCORE.lock()).get_endpoint_mut(eid) {
                 ep.clear_delivery_addr();
@@ -179,10 +177,10 @@ impl WsAASession {
                     return Ok(());
                 }
                 let recv = WsRecvData {
-                    bid: &bndl.id(),
-                    src: &bndl.primary.source.to_string(),
-                    dst: &bndl.primary.destination.to_string(),
-                    data: bndl.payload().unwrap(),
+                    bid: bndl.id(),
+                    src: bndl.primary.source.to_string(),
+                    dst: bndl.primary.destination.to_string(),
+                    data: bndl.payload().unwrap().to_vec(),
                 };
                 match format {
                     DataReceiveFormat::CBOR => {
@@ -215,7 +213,7 @@ impl WsAASession {
                         "/node" => {
                             ws_reply_text!(
                                 socket,
-                                &format!("200 node: {}", (*CONFIG.lock()).host_eid.to_string())
+                                &format!("200 node: {}", (*CONFIG.lock()).host_eid)
                             );
                         }
                         "/bundle" => {
@@ -364,8 +362,8 @@ impl WsAASession {
                             } else {
                                 BundleControlFlags::BUNDLE_MUST_NOT_FRAGMENTED
                             };
-                            let dst = EndpointID::try_from(send_req.dst);
-                            let src = EndpointID::try_from(send_req.src);
+                            let dst = EndpointID::try_from(send_req.dst.clone());
+                            let src = EndpointID::try_from(send_req.src.clone());
                             if dst.is_err() || src.is_err() {
                                 warn!(
                                     "Received data with invalid src ({}) or destination ({})",
@@ -462,10 +460,10 @@ impl WsAASession {
                                     continue;
                                 }
                                 let recv = WsRecvData {
-                                    bid: &bundle.id(),
-                                    src: &bundle.primary.source.to_string(),
-                                    dst: &bundle.primary.destination.to_string(),
-                                    data: bundle.payload().unwrap(),
+                                    bid: bundle.id(),
+                                    src: bundle.primary.source.to_string(),
+                                    dst: bundle.primary.destination.to_string(),
+                                    data: bundle.payload().unwrap().to_vec(),
                                 };
                                 match format {
                                     DataReceiveFormat::CBOR => serde_cbor::to_vec(&recv)
