@@ -1,12 +1,14 @@
 extern crate alloc;
 use core::fmt;
-use log::info;
+use log::warn;
 use serde::de::{SeqAccess, Visitor};
 use serde::ser::{SerializeSeq, Serializer};
 use serde::{de, Deserialize, Deserializer, Serialize};
 use std::collections::HashMap;
 use std::fmt::Debug;
 
+use crate::cla::CLAsAvailable;
+
 /// Struct representing the ServiceBlock used in Beacons to advertise additional services
 ///
 /// Contains two vectors,
@@ -16,7 +18,7 @@ use std::fmt::Debug;
 /// and one for user defined services
 #[derive(Debug, Clone, PartialEq)]
 pub struct ServiceBlock {
-    clas: Vec<(String, Option<u16>)>,
+    clas: Vec<(CLAsAvailable, Option<u16>)>,
     services: HashMap<u8, Vec<u8>>,
 }
 impl Default for ServiceBlock {
@@ -34,7 +36,7 @@ impl ServiceBlock {
     }
 
     /// Returns the vector of ConvergencyLayerAgents
-    pub fn clas(&self) -> &Vec<(String, Option<u16>)> {
+    pub fn clas(&self) -> &Vec<(CLAsAvailable, Option<u16>)> {
         &self.clas
     }
 
@@ -67,7 +69,7 @@ impl ServiceBlock {
                     convert.insert(*tag, message);
                 }
                 _ => {
-                    info!("Unknown Service encountered. Compare senders IPND version with this one to check for incompatibilities.");
+                    warn!("Unknown service encountered. Compare senders IPND version with this one to check for incompatibilities.");
                 }
             }
         }
@@ -81,7 +83,8 @@ impl ServiceBlock {
 
     /// This method adds a cla to the corresponding vector of a ServiceBlock
     pub fn add_cla(&mut self, name: &str, port: &Option<u16>) {
-        self.clas.push((name.to_owned(), *port))
+        self.clas
+            .push((name.parse::<CLAsAvailable>().unwrap(), *port))
     }
 
     /// This method adds a custom service to the HashMap of a ServiceBlock
@@ -89,7 +92,7 @@ impl ServiceBlock {
         self.services.insert(tag, service.to_owned());
     }
     /// This method sets the clas vector of a ServiceBlock to the one provided
-    pub fn set_clas(&mut self, clas: Vec<(String, Option<u16>)>) {
+    pub fn set_clas(&mut self, clas: Vec<(CLAsAvailable, Option<u16>)>) {
         self.clas = clas;
     }
     /// This method sets the services hashmap of a ServiceBlock to the one provided
@@ -222,8 +225,8 @@ impl std::fmt::Display for ServiceBlock {
                             counter, tag, address[0],address[1],address[2],address[3],address[4])
                 }
                 _ => {
-                    info!("Unknown Service encountered. Compare senders IPND version with this one to check for incompatibilities.");
-                    format!("")
+                    warn!("Unknown service encountered. Compare senders IPND version with this one to check for incompatibilities.");
+                    String::new()
                 }
             };
             output.push_str(str.as_str());
